/*****************************************************************************
 *
 * utouch-grail - Multitouch Gesture Recognition Library
 *
 * Copyright (C) 2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#ifndef UTOUCH_GRAIL_LOG_H_
#define UTOUCH_GRAIL_LOG_H_

#include <iostream>

namespace utouch {
namespace grail {

class NullStreamBuf : public std::streambuf {
 protected:
  virtual int overflow(int c) { return traits_type::not_eof(c); }
};

class Logger {
 public:
  enum Level {
    Dbg = -1,
    Warn = 0,
    Err = 1,
  };

  Logger();

  static std::ostream& Log(Level level);

  static Logger& instance();

  Level level() const {return static_cast<Level>(level_);};

 private:
  int level_;
  NullStreamBuf null_buf_;
  std::ostream null_ostream_;
};

} // namespace grail
} // namespace utouch

#define LOG(level) utouch::grail::Logger::Log(utouch::grail::Logger::level) \
    << "(" << __FILE__ << ":" << __func__ << ":" << __LINE__ << "): "

#endif // UTOUCH_GRAIL_LOG_H_
