/*****************************************************************************
 *
 * utouch-grail - Multitouch Gesture Recognition Library
 *
 * Copyright (C) 2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#ifndef UTOUCH_GRAIL_SLICE_H_
#define UTOUCH_GRAIL_SLICE_H_

#include <memory>
#include <set>

#include "utouch/grail.h"
#include "v3/forward.h"

struct UGSlice_ {};

namespace utouch {
namespace grail {

class UGSlice : public UGSlice_ {
 public:
  /** @internal Create a new slice for a new gesture */
  UGSlice(const SharedGesture& gesture, UFEvent event, const TouchSet& touches,
          UGGestureTypeMask recognized);
  /** @internal Create a new copy of an existing slice */
  UGSlice(const SharedUGSlice& prev, bool end = false);
  /** @internal Create a new slice by updating an existing slice */
  UGSlice(const SharedUGSlice& prev, UFEvent event, const TouchSet& touches);

  ~UGSlice();

  void CheckGestureEnd();
  UGGestureTypeMask CheckRecognition(const UGSubscription& subscription);
  UGStatus GetTouchId(unsigned int index, UFTouchId* touch_id) const;
  UGStatus GetProperty(UGSliceProperty property, void* value) const;
  void ReferenceGesture() { gesture_ref_ = gesture_.lock(); }

  uint64_t time() const { return time_; }
  const TouchSet& touches() { return touches_; }
  const UFFrame& frame() { return frame_; }
  UGGestureState state() const { return state_; }
  bool physically_ended() const { return physically_ended_; }
  void set_state(UGGestureState state) { state_ = state; }
  bool construction_finished() const { return construction_finished_; }
  void set_construction_finished() { construction_finished_ = true; }

  UGSlice(const UGSlice&) = delete;
  UGSlice& operator=(const UGSlice&) = delete;

 private:
  const std::weak_ptr<Gesture> gesture_;
  SharedGesture gesture_ref_;
  const UFEvent event_;
  UFFrame frame_;
  TouchSet touches_;
  uint64_t time_;
  UGGestureState state_;
  bool physically_ended_;
  float original_center_x_;
  float original_center_y_;
  float original_radius_;
  float original_angle_;
  float radius_;
  float angle_;
  float transform_[3][3];
  float cumulative_transform_[3][3];
  float center_of_rotation_x_;
  float center_of_rotation_y_;
  UGGestureTypeMask recognized_;
  bool construction_finished_;
  bool touch_count_changed_;

  void GetValues(const TouchSet& touches, float* x, float* y, float* radius,
                 float* angle, bool init);
  void SetTransforms();
  void SetCenterOfRotation();
  uint64_t CumulativeTime() const;
  float CumulativeDrag2() const;
  float CumulativePinch() const;
};

} // namespace grail
} // namespace utouch

#endif // UTOUCH_GRAIL_SLICE_H_
